# Author: Jan Vcelak <jvcelak@redhat.com>

require "bughunting/check/task"
require "bughunting/web/config"
require "bughunting/web/hints"
require "bughunting/web/players"
require "bughunting/web/scores"

require "sqlite3"

class WebController
	attr_reader :config
	attr_reader :tasks, :scores, :hints, :players

	def initialize(config_filename)
		@config = WebConfig.load_file config_filename
		@tasks = Task.load_dir @config["paths.tasks"]

		@database = SQLite3::Database.new @config["results.database"]
		@scores = Scores.new @database
		@hints = Hints.new @database
		@players = Players.new @database
	end

	def tasks_list(player)
		tasks = []
		task_scores = @scores.all_tasks player
		@tasks.each do |name,task|
			tasks << {
				:name => name,
				:summary => task.config["summary"],
				:language => task.config["language"],
				:difficulty => task.config["difficulty"],
				:max_score => task.config["max_score"],
				:score => task_scores[name] || 0,
			}
		end
		tasks
	end

	def game_running?
		File.exists? @config["results.running"]
	end

	def machine_name(ip)
		@config["players"][ip] || ip
	end
end
