# Author: Jan Vcelak <jvcelak@redhat.com>

require "time"

class Scores
	def initialize(database)
		@db = database
		setup_schema
	end

	def set(player, task, score)
		@db.execute \
			"INSERT OR REPLACE INTO scores (player, task, score) VALUES (?, ?, ?)",
			[player, task, score]
	end

	def total(player)
		score = @db.get_first_value \
			"SELECT SUM(score) score FROM scores WHERE player = ?",
			[player]
		score || 0
	end

	def all_tasks(player)
		result = {}
		@db.execute "SELECT task, score FROM scores WHERE player = ?",	player do |task, score|
			result[task] = score
		end
		result
	end

	def task(player, task)
		score = @db.get_first_value \
			"SELECT score FROM scores WHERE player = ? AND task = ?",
			[player, task]
		score || 0
	end

	def ranking
		@db.execute(
			"SELECT p.player, p.name, p.email, sum(s.score) sum, strftime('%s', max(s.created_at)) last
			FROM players p LEFT JOIN scores s ON p.player = s.player GROUP BY p.player
			ORDER BY sum DESC, last ASC, p.created_at ASC"
		).collect do |player, name, email, score, last_scoring|
			{
				:player => player,
				:name => name,
				:email => email,
				:score => score || 0,
				:last_scoring => last_scoring.nil? ? nil : Time.at(last_scoring.to_i),
			}
		end
	end

	private

	def setup_schema
		@db.execute "CREATE TABLE IF NOT EXISTS scores (
			id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
			created_at DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
			player VARCHAR NOT NULL,
			task VARCHAR NOT NULL,
			score TINYINT NOT NULL,
			UNIQUE (player, task)
		)"
	end
end
